@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 ### System ###
 import csv
 
@@ -9,7 +10,7 @@ from .events import csv_to_midi_map
 COMMENT_DELIMITERS = ("#", ";")
 
 
-def parse(file):
+def parse(file, strict=True):
     """Parses a CSV file into MIDI format.
 
     Args:
